#include "bsp_uart.h"
#include "lwrb.h"
#include "CH58x_uart.h"
#include "shell_port.h"

#include "SLEEP.h"

#include "log.h"

#undef LOG_ENABLE
#define LOG_ENABLE 1

#define LOG_LVL    LOG_DEBUG

#undef LOG_TAG
#define LOG_TAG "uart"

// https://www.cnblogs.com/iot-fan/p/14744671.html

#define SHELL_TIMEOUT_EVT      (0x0001 << 0)

static tmosTaskID shell_task_id = INVALID_TASK_ID;

void BSP_Shell_SetActive(void);

// AT ML307
#define UART1_RX_BUFFER_LENGTH  1024U
#define UART1_TX_BUFFER_LENGTH  1024U

// SHELL
#define UART3_RX_BUFFER_LENGTH 128U
#define UART3_TX_BUFFER_LENGTH 1024U

lwrb_t Uart1Rx;
lwrb_t Uart1Tx;
uint8_t uart1_tx_buf[UART1_TX_BUFFER_LENGTH] = {0};
uint8_t uart1_rx_buf[UART1_RX_BUFFER_LENGTH] = {0};

lwrb_t Uart3Rx;
lwrb_t Uart3Tx;
uint8_t uart3_rx_buf[UART3_RX_BUFFER_LENGTH] = {0};
uint8_t uart3_tx_buf[UART3_TX_BUFFER_LENGTH] = {0};

// ڽ
unsigned int BSP_Uart1_Receive_Data(void *buf, unsigned int len)
{
    return lwrb_read(&Uart1Rx, buf, len);
}

unsigned int BSP_Uart3_Receive_Data(void *buf, unsigned int len)
{
    return lwrb_read(&Uart3Rx, buf, len);
}

// жϷ
unsigned int BSP_Uart1_Send_Data(const void *buf, unsigned int len)
{
    unsigned int ret;

    ret = lwrb_write(&Uart1Tx, buf, len);
    UART1_INTCfg(ENABLE, RB_IER_THR_EMPTY);
    // BSP_BlockSleep();
    // BSP_UART1_TxLoop();
    // BSP_RequestSleep();
    return ret;
}

unsigned int BSP_Uart1_Send_String(const void *buf)
{
    return BSP_Uart1_Send_Data(buf, strlen((const char *)buf));
}

// https://www.cnblogs.com/iot-fan/p/13439293.html
// tx process
// ѯҪӵѭϵ
void BSP_UART3_TxLoop(void)
{
    uint8_t data;
    while (R8_UART3_TFC < UART_FIFO_SIZE)
    {
        // жϷ,Ƿ,,һһ,Ӳfifo
        if (lwrb_get_full(&Uart3Tx))
        {
            lwrb_read(&Uart3Tx, &data, 1);
            // uartӲfifo
            UART3_SendByte(data);
        }
        else
        {
            break;
        }
    }
}

// ڷ
unsigned int BSP_Uart3_Send_Data(const void *buf, unsigned int len)
{
    unsigned int ret;

    ret = lwrb_write(&Uart3Tx, buf, len);
    // UART3_INTCfg(ENABLE, RB_IER_THR_EMPTY);

    // BSP_BlockSleep();
    BSP_UART3_TxLoop();
    // BSP_RequestSleep();
    return ret;
}

void UART1_FifoInit(void)
{
    lwrb_init(&Uart1Tx, uart1_tx_buf, sizeof(uart1_tx_buf));
    lwrb_init(&Uart1Rx, uart1_rx_buf, sizeof(uart1_rx_buf));
}

void UART3_FifoInit(void)
{
    lwrb_init(&Uart3Tx, uart3_tx_buf, sizeof(uart3_tx_buf));
    lwrb_init(&Uart3Rx, uart3_rx_buf, sizeof(uart3_rx_buf));
}

void BSP_UART1_Init(unsigned int baudrate)
{
    GPIOPinRemap(ENABLE, RB_PIN_UART1);

    /* ô1IOģʽô */
    GPIOB_SetBits(bTXD1_);
    GPIOB_ModeCfg(bRXD1_, GPIO_ModeIN_PU);      // RXD-
    GPIOB_ModeCfg(bTXD1_, GPIO_ModeOut_PP_5mA); // TXD-עIOߵƽ
    // UART1_DefInit();
    {
        UART1_BaudRateCfg(baudrate);
        R8_UART1_FCR = (2 << 6) | RB_FCR_TX_FIFO_CLR | RB_FCR_RX_FIFO_CLR | RB_FCR_FIFO_EN; // FIFO򿪣4ֽ
        R8_UART1_LCR = RB_LCR_WORD_SZ;
        R8_UART1_IER = RB_IER_TXD_EN;
        R8_UART1_DIV = 1;
    }
    UART1_ByteTrigCfg(UART_1BYTE_TRIG);

    // жϷʽ
    UART1_INTCfg(ENABLE, RB_IER_LINE_STAT | RB_IER_RECV_RDY | RB_IER_THR_EMPTY);
    PFIC_EnableIRQ(UART1_IRQn);

    UART1_FifoInit();
}

void BSP_UART3_Init(uint32_t baudrate)
{
    GPIOPinRemap(ENABLE, RB_PIN_UART3);

    /* ô3IOģʽô */
    // TODO:ע bTXD3_
    GPIOB_SetBits(GPIO_Pin_21);
    GPIOB_ModeCfg(GPIO_Pin_20, GPIO_ModeIN_PU);      // RXD-
    GPIOB_ModeCfg(GPIO_Pin_21, GPIO_ModeOut_PP_5mA); // TXD-עIOߵƽ
    // UART3_DefInit();
    UART3_BaudRateCfg(baudrate);
    R8_UART3_FCR = (2 << 6) | RB_FCR_TX_FIFO_CLR | RB_FCR_RX_FIFO_CLR | RB_FCR_FIFO_EN; // FIFO򿪣4ֽ
    R8_UART3_LCR = RB_LCR_WORD_SZ;
    R8_UART3_IER = RB_IER_TXD_EN;
    R8_UART3_DIV = 1;
    UART3_ByteTrigCfg(UART_1BYTE_TRIG);

    // жϷʽ
    UART3_INTCfg(ENABLE, RB_IER_LINE_STAT | RB_IER_RECV_RDY | RB_IER_THR_EMPTY);
    PFIC_EnableIRQ(UART3_IRQn);

    UART3_FifoInit();
}

/**
 * @brief ʱϵͳ˯
 */
static uint16_t BSP_Shell_ProcessEvent(uint8_t task_id, uint16_t events)
{
    if (events & SHELL_TIMEOUT_EVT)
    {
        logDebug("SHELL_TIMEOUT_EVT UART3");
        BSP_RequestSleep();
        return (events ^ SHELL_TIMEOUT_EVT);
    }

    return 0;
}

/**
 * @brief Shellʱʱ
 */
void BSP_Shell_RefreshTimeoutTimer(void)
{
    tmos_stop_task(shell_task_id, SHELL_TIMEOUT_EVT);
    tmos_start_task(shell_task_id, SHELL_TIMEOUT_EVT, MS1_TO_SYSTEM_TIME(SHELL_ACTIVITY_TIMEOUT));
}

/**
 * @brief ShellΪ״ֹ̬ϵͳ˯
 */
void BSP_Shell_SetActive(void)
{
    BSP_BlockSleep();
    BSP_Shell_RefreshTimeoutTimer();
}

void BSP_SHELL_Init(uint32_t baudrate)
{
    BSP_UART3_Init(baudrate);
    SHELL_Init();
    if (shell_task_id == INVALID_TASK_ID)
    {
        shell_task_id = TMOS_ProcessEventRegister(BSP_Shell_ProcessEvent);
        logDebug("Shell task id: %d", shell_task_id);
    }
}

/*********************************************************************
 * @fn      UART1_IRQHandler
 *
 * @brief   UART1жϺ
 *
 * @return  none
 */
__INTERRUPT
__HIGH_CODE
void UART1_IRQHandler(void)
{
    uint8_t data;

    switch (UART1_GetITFlag())
    {
        case UART_II_LINE_STAT: // ·״̬
        {
            logError("R8_UART1_LSR = 0x%X", UART1_GetLinSTA());
            break;
        }
        case UART_II_RECV_RDY:
        {
            data = UART1_RecvByte();
            lwrb_write(&Uart1Rx, &data, 1);
            break;
        }
        case UART_II_RECV_TOUT: // ճʱ
            break;
        case UART_II_THR_EMPTY: // ͻգɼ;жϷͻӰ
        {
            if (lwrb_read(&Uart1Tx, &data, 1))
            {
                UART1_SendByte(data);
            }
            else
            {
                UART1_INTCfg(DISABLE, RB_IER_THR_EMPTY);
            }
            break;
        }
        break;

        case UART_II_MODEM_CHG: // ִֻ֧0
            break;

        default:
            break;
    }
}

/*********************************************************************
 * @fn      UART3_IRQHandler
 *
 * @brief   UART3жϺ
 *
 * @return  none
 */
__INTERRUPT
__HIGH_CODE
void UART3_IRQHandler(void)
{
    uint8_t data;

    switch (UART3_GetITFlag())
    {
        case UART_II_LINE_STAT: // ·״̬
        {
            logError("R8_UART3_LSR = 0x%X", UART3_GetLinSTA());
            break;
        }
        case UART_II_RECV_RDY:
        case UART_II_RECV_TOUT: // ճʱ
        {
            BSP_Shell_SetActive();
            while (R8_UART3_RFC)
            {
                shellHandler(&shell, R8_UART3_RBR);
            }
            // logDebug("UART_II_RECV_TOUT");
            break;
        }
        case UART_II_THR_EMPTY: // ͻգɼ
            // жϷͻӰ
            break;

        case UART_II_MODEM_CHG: // ִֻ֧0
            break;

        default:
            break;
    }
}
